(function () {

	"use strict";

	/*
		Maybe is a set consisting of two directions, Just a value and Nothing.
		Represents the existence or not of a value.
	*/

	Ext.define("OPSupport.Maybe", function() {
		var Nothing = {
			x: "Nothing",
			map: function(f) { return Nothing; },
			chain: function(f) { return Nothing; },
			ap: function(other) { return Nothing; },
			fold: function(f, g) { return f(); }
		};

		var Just = function(x) {
			return {
				x: x,
				map: function(f) { return validate(x) ? Just(f(x)) : Nothing; },
				chain: function(f) { return f(x); },
				ap: function(other) { return other.map(x); },
				fold: function(f, g) { return g(x); }
			};
		};

		// validate : a -> Bool
		var validate = function(x) {
			return (x !== null) && (x !== undefined);
		};

		// of : a -> Maybe a
		var of = function(x) {
			return (validate (x)) ? Just(x) : Nothing;
		};

		// widhtDefault : b -> Maybe a -> a | b
		var withDefault = function(x, ma) {
			return ma.fold(always(x), identity);
		};

		// isNothing : Maybe a -> Bool
		var isNothing = function(ma) {
			return ma.fold
				( always(true)
				, always(false)
				);
		};

		// testEmpty : a -> Maybe a
		var testEmpty = function(x) {
			return compose
				( [ map(always(x))
				  , chain(boolToMaybe)
				  , map(isNotEmpty)
				  , of
				  ]
                , x
                );
		};

        var always = function(x) {
            return function() { return x; };
        };

        var identity = function(x) { return x; };

        var compose = function(xs, init) {
            return xs.reduceRight
                ( function(value, f) { return f(value); }
                , init
                )
        }

        var map = function(f) {
            return function(ma) {
                return ma.map(f);
            };
        };

        var chain = function(f) {
            return function(ma) {
                return ma.chain(f);
            };
        };

        // boolToMaybe : Bool -> Maybe Bool
		var boolToMaybe = function(flag) {
            return (flag === true)
                ? installwiz.Maybe.Just(true)
                : installwiz.Maybe.Nothing;
        };

        // isNotEmpty : a -> Bool
	    var isNotEmpty = function(x) {
			return (x !== "") && (x.length && x.length > 0);
		};

        // head : [a] -> Maybe a
        var head = function(arr) {
            return of(arr[0]);
        };

        // pluck : [String] -> Object a -> Maybe a
        var pluck = function(xs) {
            return function(o) {
                return xs.reduce(
                        function(ma, key) { return ma.chain(function(o) { return of (o[key]); }); },
                        of(o)
                        );
            };
        };

		return {
			singleton: true,
			Nothing: Nothing,
			Just: Just,
			of: of,
			withDefault: withDefault,
			isNothing: isNothing,
			testEmpty: testEmpty,
            head: head,
            pluck: pluck
		};
	});

}) ();
