#!/usr/bin/php -q
<?php

$locale = null;
$num_translated = $num_missed = 0;
$changed_files = array();
$missed_strings = array();
$missed_strings_sql = array();
$missing_file_sql = 'missing.sql';
$use_google_to_translate_strings_oh_yeah=false;
$locale_server = 'opmon.opservices.com.br';
$locale_user = 'opmonadmin';
$locale_password = 'y0ub3tt3r73?41c4lls4ul';

$options = getopt("d:l::htg",array('dir:','locale::','help','test','google'));

$usage = <<<EOF
Script to convert language macros "{{{STRING}}" to Ext.locale.translate.

Usage: {$argv[0]} -d extjs_project_dir
	Options:
	-h,--help 	this help.
	-d,--dir 	extjs_project_dir 		directory containing extjs project.
	-t,--test 	testing 			dry run without changing files.
	-l,--locale 	[locale_file]			Use locale file with translations in json format (/opmon/oplib/libphp/lang.php?f=json) as reference for what's already translated or not.
	-g,--google 	use google to translate

Example: Translate files from current project folder using locale.json as reference and google translator as helper.
# cd ng; $argv[0] -d . -l -g

EOF;


if (isset($options['h']) || !isset($options['d']) || (isset($options['d']) && !is_dir($options['d']))) {
	print $usage;
	exit;
}

if (isset($options['g'])) {
    $use_google_to_translate_strings_oh_yeah = true;
}
print "Starting translations\n";
//use locale
if (isset($options['l'])) {
	$locale_file = null;
	if (is_file($options['l'])) {
		$locale_file = $options['l'];
	} elseif (is_file($options['d'].'/locale.json')) {
		$locale_file = $options['d'].'/locale.json';
	} else {
		print "Downloading locale.json from server $locale_server.\n";
		system('wget --quiet --user='.$locale_user.' --password='.$locale_password.' --auth-no-challenge --no-check-certificate -nc -O '.$options['d'].'/locale.json "https://'.$locale_server.'/opmon/oplib/libphp/lang.php?f=json"');
		$locale_file = $options['d'].'/locale.json';
	}
	print "Using ".$locale_file."\n";
	if (!is_file($locale_file) || strlen(file_get_contents($locale_file)) < 100) {
		print "Error: Locale file not found or too short.\n";
		exit(1);
	}

	$locale = json_decode(file_get_contents($locale_file));
}

function gtranslate($str,$from='en_US',$to='pt_BR') {
	print "Translating string $str\n";
	$options = array(
		'http'=>array(
		'method'=>"GET",
		'header'=>"Accept-language: en\r\n" .
		          "Cookie: foo=bar\r\n" .
		          "User-Agent: Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.102011-10-16 20:23:10\r\n"
		)
	);
	$url = 'https://translate.googleapis.com/translate_a/single?client=gtx&sl='.$from.'&tl='.$to.'&dt=t&q='.urlencode($str);
	$context = stream_context_create($options);
	$file = explode('"',file_get_contents($url, false, $context));
	return isset($file[1]) ? $file[1] : $str;
}

function replacement_callback($matches) {
	global $locale,$options,$num_translated,$num_missed,$missed_strings,$missed_strings_sql;
	$m = $matches[1];
	if ($locale) {
		print "Checking string: $m\n";
		$p_pattern = array('/\.\.\.$/','/\.$/','/:$/','/ \?$/','/\?$/','/ $/');
		foreach($p_pattern as $p) {
			if (!$p_match && preg_match($p, $m, $pm)) {
				$p_match = isset($pm[0]) ? $pm[0] : '';
				$m = preg_replace($p,'',$m);
			}
		}

		if (!isset($locale->translation->{$m})) {
			$missed_strings[] = $m;
			$num_missed++;
		} else {
			$num_translated++;
		}

	}
	return "Ext.locale.translate('".$m."')";
}


function replacement_callback_alt($matches) {
	return "'+".replacement_callback($matches)."+'";
}

function rscandir($dir) {
	global $options,$changed_files;
	$dh = opendir($dir);
	while (false !== ($filename = readdir($dh))) {
		if (in_array($filename,array('.','..'))) continue;
		$filename = $dir . DIRECTORY_SEPARATOR . $filename;
		if (is_dir($filename)){
			rscandir($filename);
		} else if(substr($filename, -3) === '.js') {
			$content = file_get_contents($filename);
			$patterns = "/['\"]\{\{(.*?)\}\}['\"]/";
			if ($new_content = preg_replace_callback($patterns, 'replacement_callback', $content)) {
				if (strcmp($content, $new_content) !== 0) {
					$changed_files[] = $filename;
					if (!isset($options['t'])) {
						file_put_contents($filename, $new_content);
					}
				}
			}

			$patterns = "/\{\{(.*?)\}\}/";
			if ($content = preg_replace_callback($patterns, 'replacement_callback_alt', $new_content)) {
				if (strcmp($new_content, $content) !== 0) {
					$changed_files[] = $filename;
					if (!isset($options['t'])) {
						file_put_contents($filename, $content);
					}
				}
			}


		}
	}
	closedir($dh);
}

if (isset($options['t'])) {
	print "Dry run\n";
}

rscandir($options['d'].'/');

$num_total = $num_translated + $num_missed;
$summary = <<<EOF
Summary:
%d file(s) translated.
%s
%s

EOF;

//only use advanced features if locale exists
if ($locale) {
	$lcache = array();
	$missed_strings_sql[] = "USE opcfg;\n";
	foreach (array('pt_BR','pt_BR_ITIL','en_US') as $lang) {
		foreach (array_unique($missed_strings) as $value) {
			$tvalue = $value;
			if ($lang == 'pt_BR') {
				if (isset($lcache[$value])) {
					$tvalue = $lcache[$value];
				} else {
					$lcache[$value] = $use_google_to_translate_strings_oh_yeah ? gtranslate($value,'en_US','pt_BR') : $value;
					$tvalue = $lcache[$value];
				}
			} else if ($lang == 'pt_BR_ITIL') {
				$tvalue = $lcache[$value];
			}
			$missed_strings_sql[] = "INSERT INTO languages VALUES ('".$lang."','".addslashes($value)."','".htmlentities($tvalue)."');";
		}
	}

	$locale_summary = "\n$num_translated translated string(s) of $num_total\n\n$num_missed missed string(s) of $num_total\n\nMissed string(s):\n".implode("\n",$missed_strings);
	$locale_summary .= "\n\nMissing strings sql file write to: $missing_file_sql\n";
	file_put_contents($missing_file_sql, implode("\n", $missed_strings_sql)."\n");
} else {
	$locale_summary = '';
}

printf($summary,count($changed_files),implode("\n",$changed_files),$locale_summary);
?>
