(function(w, Ext) {

    /**
     * Functions Utils
     */

    function curry(fx) {
        var arity = fx.length;
        return function f1() {
            var args = Array.prototype.slice.call(arguments, 0);
            return (args.length >= arity)
            ? fx.apply(null, args)
            : function() {
                var args2 = Array.prototype.slice.call(arguments, 0);
                return f1.apply(null, args.concat(args2));
            };
        };
    }

    // compose :: [(a -> b)] -> a -> b
    function compose(fs) {
        return function(init) {
            return fs.reduceRight(function(res, f) { return f(res); }, init);
        }
    }

    // pipe :: [(a -> b)] -> a -> b
    function pipe(fs) {
        return function(init) {
            return fs.reduce(function(res, f) { return f(res); }, init);
        }
    }

    // always :: a -> _ -> a
    var always = curry(function(x, _) { return x; })
    
    // K :: a -> _ -> a
    var K = function(x) {
        return function() {
            return x;
        }
    }

    // identity :: a -> a
    function identity(x) { return x; }

    // log :: String -> a -> a
    var log = curry(function(msg, x) {
        return (console.warn("## LOG ## " + msg + ": ", x), x);
    });

    // equals :: a -> b -> Bool
    var equals = curry(function(x, y) { return (x === y); });

    // diff :: a -> b -> Bool
    var diff = curry(function(x, y) { return !equals (x, y); });

    // and :: a -> b -> Bool
    var and = curry(function(x, y) { return x && y; });

    // or :: a -> b -> Bool
    var or = curry(function(x, y) { return x || y; });

    w.op.Functions =
        { compose: compose
        , pipe: pipe
        , curry: curry
        , always: always
        , identity: identity
        , log: log
        , equals: equals
        , diff: diff
        , and: and
        , or: or
        , K: K
        }


    /**
     * Object Utils
     */

    // assign :: String -> a -> Object a -> Object a
    var assign = curry(function(key, value, o) {
        var fresh = Ext.clone(o);
        fresh[key] = value;
        return fresh;
    });

    // pluck :: String -> Object a -> Maybe a
    var pluck = curry(function(key, o) {
        return w.op.Maybe.of (o [key]);
    });

    // deepPluck :: [String] -> Object a -> a
    var deepPluck = curry(function(ks, object) {
        return ks.reduce(function(ma, k) {
            return ma.chain (pluck (k));
        }, w.op.Maybe.of (object));
    });

    w.op.Object =
        { assign: assign
        , pluck: pluck
        , deepPluck: deepPluck
        }


    /**
     * List Utils
     */

    // removeFirstLast :: List a -> List a
    function removeFirstLast(str) {
        return str.slice(1).slice(0,-1);
    }

    // last :: List a -> a
    function last(xs) {
        return xs.slice(-1);
    }

    // head :: List a -> Maybe a
    function head(xs) {
        return w.op.Maybe.of (xs [0]);
    }

    // all :: (a -> Bool) -> [a] -> Bool
    var all = curry(function(f, xs) {
        return xs.reduce(function(acc, x) {
            return acc && f (x);
        }, true);
    });

    // any :: (a -> Bool) -> [a] -> Bool
    var any = curry(function(f, xs) {
        return xs.reduce(function(acc, x) {
            return acc || f (x);
        }, false);
    });

    // zip :: (...a -> b) -> ...[a] -> [[b]]
    var zip = function(f) {
        return function() {
            var xxs = Array.prototype.slice.call(arguments);
            return lowerArray(xxs)
                .map(function(_, i) {
                    var args = xxs.map(function(xs) { return xs[i]; });
                    return f.apply(null, args);
                });
        };
    };

    // lowerArray :: [[a]] -> [a]
    var lowerArray = function(xxs) {
        return xxs.reduce(function(lower, next) {
            return next.length < lower.length ? next : lower;
        });
    };

    // head :: [a] -> Maybe a
    var head = function(xs) {
        return op.Maybe.of(xs[0]);
    }

    w.op.List =
        { removeFirstLast: removeFirstLast
        , last: last
        , head: head
        , all: all
        , any: any
        , zip: zip
        , lowerArray: lowerArray
        , head: head
        }
    

    /**
     * String Util
     */

    // trim :: String -> String
    function trim(str) {
        return str.trim();
    }

    // split :: Regex -> String -> [String]
    var split = curry(function(reg, str) {
        return str.split (reg);
    });

    w.op.String =
        { trim: trim
        , split: split
        }

    var getTwoHours = function() {
        var d = new Date();
        d.setHours(d.getHours() + 2);
        return d;
    }
    
    var formatDateToDb = function(d) {
        return Ext.Date.format(new Date(d), 'Y-m-d')
    };
    
    var formatTimeToDb = function(d) {
        return Ext.Date.format(new Date(d), 'H:i:s')
    };

    var formatDateEN = function(d) {
        return Ext.Date.format(new Date(d), 'm/d/Y')
    };
    
    var formatDatePT = function(d) {
        return Ext.Date.format(new Date(d), 'd/m/Y')
    };

    
    w.op.Date =
        { getTwoHours: getTwoHours,
          formatDateToDb: formatDateToDb,
          formatTimeToDb: formatTimeToDb,
          formatDateEN: formatDateEN,
          formatDatePT: formatDatePT
        }
}) (window, (window.Ext || parent.Ext));
