(function(w) {

    /**
     * Type Either a | b
     * | Left b
     * | Right a
     */

    function handleErr(f, g, x) {
        try {
            return g (x);
        } catch (e) {
            //console.warn("OP LOG INFO", e);
            return f (e);
        }
    }

    function exec(f, x) {
        return handleErr(Left, function(y) { return Right (f (y)); }, x);
    }

    var Left = function(x) {
        return (
            { x: x
            , isRight: false
            , map: function(f) { return Left (x); }
            , chain: function(f) { return Left (x); }
            , ap: function(other) { return Left (x); }
            , fold: function(f, g) { return f (x); }
            }
        );
    };

    var Right = function(x) {
        return (
            { x: x
            , isRight: true
            , map: function(f) { return exec (f,x); }
            , chain: function(f) { return exec(f,x).fold(Left, function(y) { return y; }); }
            , ap: function(other) { return other.map(x); }
            , fold: function(f, g) { return handleErr(f, g, x); }
            }
        );
    };

    // of :: a -> Either a
    var of = Right;

    // eitherToTask :: Either a -> Task a
    function eitherToTask(either) {
        return either.fold
            ( function(e) { return Task.rejected(e); }
            , function(x) { return Task.of(x); }
            );
    }

    w.op.Either =
        { Left: Left
        , Right: Right
        , of: of
        , eitherToTask: eitherToTask
        , execOf: exec
        };
    
}) (window);



(function(w) {

    /**
     * Type Maybe a
     * | Nothing
     * | Just a
     */

    var Nothing =
        { x: "Nothing"
        , isNothing: true
        , map: function(f) { return Nothing; }
        , chain: function(f) { return Nothing; }
        , ap: function(other) { return Nothing; }
        , fold: function(f, g) { return f (); }
        };

    var Just = function(x) {
        return (
            { x: x
            , isNothing: false
            , map: function(f) { return validate(x) ? Just (f (x)) : Nothing; }
            , chain: function(f) { return validate(x) ? f (x) : Nothing; }
            , ap: function(other) { return other.map(x); }
            , fold: function(f, g) { return validate (x) ? g (x) : f(); }
            }
        );
    };

    // validate :: a -> Bool
    function validate(x) {
        return (x !== null) && (x !== undefined);
    }

    // of :: a -> Maybe a
    function of(x) {
        return validate (x)
            ? Just (x)
            : Nothing
    }

    // withDefault :: b -> Maybe a -> a | b
    var withDefault = w.op.Functions.curry(function(x, ma) {
        return ma.fold(w.op.Functions.always(x), w.op.Functions.identity);
    });

    // maybeToTask :: Maybe a -> Task a
    function maybeToTask(ma) {
        return ma.fold
            ( function(e) { return Task.rejected(e); }
            , function(x) { return Task.of(x); }
            );
    }

    w.op.Maybe =
        { Nothing: Nothing
        , Just: Just
        , of: of
        , withDefault: withDefault
        , maybeToTask: maybeToTask
        };

}) (window);

(function(w) {

    var Operator = function() {
        return {
            isAdmin: false,
            fold: function(obj) {
                return obj.Operator();
            }
        };
    };

    var Admin = function() {
        return {
            isAdmin: true,
            fold: function(obj) {
                return obj.Admin();
            }
        };
    };

    var OpAdmin = function() {
        return {
            isAdmin: true,
            fold: function(obj) {
                return obj.OpAdmin();
            }
        };
    };

    var of = function(response) {
        return (!response || response.type === "2")
            ? Operator()
            : (response.type === "0" && response.contact_name === "opmonadmin") ? OpAdmin() : Admin();
    };

    w.op.UserType = {
        Operator: Operator,
        Admin: Admin,
        OpAdmin: OpAdmin,
        of: of
    };

})(window);