(function()
    {
        "use strict"

        Ext.define("OPSupport.Observer", {
            constructor: function(ids) {
                var me = this;
                var handlers =
                    [].concat(ids || []).reduce(reduceHandlers, {});

                this.trigger = function(id, xs, ctx) {
                    return (propertyNotExist(handlers, id))
                        ? warnId(id)
                        : execute(handlers[id], [].concat(xs), ctx);
                }

                this.getHandlers = function(id) {
                    return (id) ? handlers [id] : handlers;
                }

                this.subscribe = function(id, f) {
                    if (typeof f !== "function") return (warnFn(), me);
                    if (propertyNotExist(handlers, id)) return (warnId(id), me);
                    return (handlers[id].push(f), me);
                }

                this.on = this.subscribe;
            }
        });


        function reduceHandlers(acc, id) {
            return (acc.hasOwnProperty(id))
                ? acc
                : (acc[id] = [], acc);
        }

        function execute(xs, args, ctx) {
            return xs.reduce
                    ( function(acc, f) { return f.apply(ctx, args); }
                    , null
                    )
        }

        function propertyNotExist(object, key) {
            return (object.hasOwnProperty(key) === false);
        }

        function warnId(x) {
            return (console.warn("ID " + x + " doesnt exist in observer."), null);
        }

        function warnFn() {
            return (console.warn("Not passing a function to subscribe."), null);
        }

    }
)();