Ext.define("SNMPCommunity", {
	extend: "Ext.data.Model",
	fields: [
		{name: "community_id", type: "int"},
		{name: "snmp_version", type: "string", defaultValue: "2c"},
		{name: "community", type: "string" },
		{name: "authentication_protocol", type: "string", defaultValue: "SHA"},
		{name: "authentication_passphrase", type: "string"},
		{name: "security_level", type: "string", defaultValue: "authPriv"},
		{name: "context_name", type: "string"},
		{name: "security_username", type: "string"},
		{name: "privacy_protocol", type: "string", defaultValue: "AES"},
		{name: "privacy_protocol_passphrase", type: "string"},
		{name: "destination_engine", type: "string"},
		{name: "active", type: "int", defaultValue: 1}
	],
	statics: {
		valid_snmp_versions: ["1", "2c", "3"],
		valid_authentication_protocols: ["SHA","MD5"],
		valid_security_levels: ["authPriv", "authNoPriv", "noAuthNoPriv"],
		valid_privacy_protocols: ["AES","DES"]
	},
	idProperty: "community_id",
	validations: [
		{type: 'presence', field: 'snmp_version'},
		{type: 'inclusion', field: 'snmp_version', list: ["1", "2c", "3"]},
		{type: 'inclusion', field: 'authentication_protocol', list: ["SHA","MD5"]},
		{type: 'inclusion', field: 'security_level', list: ["authPriv", "authNoPriv", "noAuthNoPriv"]},
		{type: 'inclusion', field: 'privacy_protocol', list: ["AES","DES"]}
	]
});

Ext.define("SNMPCommunityStore", {
	extend: "Ext.data.Store",
	model: "SNMPCommunity",
	callbackObject: null,
	callback: null,
	proxy: {
		type: "ajax",
		format: "json",

		api: {
			create: "/opmon/seagull/www/index.php/discovery/action/communities/?operation=save",
			read: "/opmon/seagull/www/index.php/discovery/action/communities/",
			update: "/opmon/seagull/www/index.php/discovery/action/communities/?operation=save",
			destroy: "/opmon/seagull/www/index.php/discovery/action/communities/?operation=delete"
		},

		reader: {
			root: "communities",
			successProperty: "success"
		},

		writer: {
			type: "json",
			writeAllFields: true,
			allowSingle: false
		}

	},
	listeners: {
		load: function(me) {
			if (!me.callbackObject || !me.callback) {
				_err("SNMPCommunityStore: undefined callback");
				return;
			}

			if (typeof(this.callbackObject[this.callback]) != "function") {
				_err("SNMPCommunityStore: invalid callback");
				return;
			}

			this.callbackObject[this.callback](me);
		}
	},
	setCallback: function(obj, func) {
		this.callbackObject = obj;
		this.callback = func;
	}
});
